#*2008.09.23 13:51
# think Tag needs to be abstracted to association
# maybe make it a subclass of dict
# so then keys are just the different tags
# and the items will be lists of times (pointing to entries)

#this is the format used to index entries based on time in the journal dict
#time_format = "%Y.%m.%d %H:%M"
#would be nice if we could just use a timestamp directly here.
from timestamp import Timestamp

import re

def check_ignore(item, ignores=[]):
    ignore = False
    for i in ignores:
        if i and re.search(i, item):
            #print "ignoring item: %s for ignore: %s" % (item, i)
            ignore = True
    return ignore

class Association(dict):
    """
    Object to hold dict of tags as keys, and the list of times as items
    """
    def has_entry(self, entry):
        """
        return a list of keys that have that entry
        """
        matches = []
        for k in self.keys():
            if self.key_has_entry(k, entry):
                matches.append(k)

        return matches
            
    def key_has_entry(self, key, entry):
        """
        look at only one association
        """
        #text_time = time_to_text(entry.created_at)
        #text_time = entry.created_at.strftime(time_format)
        text_time = str(Timestamp(time=entry.created_at))

        if self.has_key(key) and text_time in self[key]:
            return True
        else:
            return False

    def associate_entry(self, entry, key):
        #text_time = entry.created_at.strftime(time_format)
        #text_time = time_to_text(entry.created_at)
        text_time = str(Timestamp(time=entry.created_at))

        if self.has_key(key):
            if not self.key_has_entry(key, entry):
                self[key].append(text_time)
                self[key].sort()
        else:
            self[key] = [ text_time ]


    def remove_association(self, entry, key):
        text_time = str(Timestamp(time=entry.created_at))
        #text_time = entry.created_at.strftime(time_format)
        if self.key_has_entry(key, entry):
            #text_time = time_to_text(entry.created_at)
            self[key].remove(text_time)

    def remove_entry(self, entry):
        for k in self.keys():
            self.remove_association(entry, k)

    def frequency_list(self):
        """
        make a list of all unique items
        and how many times that item shows up in the journal

        return a list of tuples:
        [ (freq, item), ... ]
        """
        items = []
        for key in self.keys():
            items.append( (len(self[key]), key) )
        return items

    def max_key(self, ignores=[]):
        print ignores
        maxcount = 1
        maxkey = ''
        for key in self.keys():
            ignore = check_ignore(key, ignores)
            if not ignore:
                count = len(self[key])
                if count >= maxcount:
                    maxkey = key
                    maxcount = count
        return maxkey

    def min_key(self):
        ## mincount = maxcount
        ## for key in self.keys():
        ##     count = len(self[key])
        ##     mincount = min(count, mincount)

        mincount = None
        minkey = ''
        for key in self.keys():
            count = len(self[key])
            if not minkey or count < mincount:
                minkey = key
                mincount = count
        return minkey

    def make_cloud(self, url_template='%s', ignores=[]):
        """
        take an association, create a cloud of the keys in the association
        based on the number of items associated with each key

        inspired by:
        timfanelli.com/projects/tag_cloud.txt
        """
        cloud = ''
        maxkey = self.max_key(ignores)
        maxcount = len(self[maxkey])
        minkey = self.min_key()
        mincount = len(self[minkey])
        distribution = ( maxcount - mincount ) / 6
        for key in self.keys():
            if not check_ignore(key, ignores):
                if ( len(self[key]) == maxcount ):
                    size = "largestTag"
                elif len(self[key]) > ( mincount + (distribution*5) ):
                    size = "largerTag"
                elif len(self[key]) > ( mincount + (distribution*4) ):
                    size = "largeTag"
                elif len(self[key]) > ( mincount + (distribution*3) ):
                    size = "mediumTag"
                elif len(self[key]) > ( mincount + (distribution*2) ):
                    size = "smallTag"
                elif len(self[key]) > ( mincount + (distribution) ):
                    size = "smallerTag"
                elif len(self[key]) > mincount:
                    size = "tinyTag"
                elif len(self[key]) == mincount:
                    size = "tiniestTag"
                cloud += "<a href='%s' class='%s' alt='There are %s entries tagged as %s'>%s</a>\n" % ( url_template % key, size, str(len(self[key])), key, key )
        return cloud

    def make_log_cloud(self, steps):
        """
        http://www.car-chase.net/2007/jan/16/log-based-tag-clouds-python/
        """
        input = []
        for key in self.keys():
            input.append( (key, len(self[key])) )
            if not type(input) == types.ListType or len(input) <= 0 or steps <= 0:  
                raise InvalidInputException, "Please be sure steps > 0 and your input list is not empty."  
            else:  
                temp, newThresholds, results = [], [], []  
                for item in input:  
                    if not type(item) == types.TupleType:  
                        raise InvalidInputException, "Be sure input list holds tuples."  
                    else:
                        temp.append(item[1])  
        Weight = float(max(temp))  
        minWeight = float(min(temp))  
        newDelta = (maxWeight - minWeight)/float(steps)  
        for i in range(steps + 1):  
            newThresholds.append((100 * math.log((minWeight + i * newDelta) + 2), i))  
            for tag in input:  
                fontSet = False  
                for threshold in newThresholds[1:int(steps)+1]:  
                    if (100 * math.log(tag[1] + 2)) <= threshold[0] and not fontSet:  
                        results.append(dict({str(tag[0]):str(threshold[1])}))  
                        fontSet = True  
        return results  
