from datetime import datetime
import string, re

#from webhelpers.html import url_escape
#or use urllib.quote(path)
import urllib

from timestamp import Timestamp
from tags import Tags

class Entry:
    """
    Object to hold a unique Journal Entry
    """
    #is it ok to default to a time_created of now? (instead of none)
    def __init__(self, data=u'', tags=[], created=datetime.now(), closed=None, placeholder=False):

        self.data = data
        
        #*2009.03.06 10:56:36 
        #considering a refactor to just "created" and "closed"
        #these are referenced extensively in the moments module
        #and probably elsewhere too
        self.created_at = created
        self.closed_at = closed
        self.tags = Tags(tags)
        
        #should not be stored in any database
        self.placeholder = placeholder

        #2008.07.05 09:22
        #context should be the filename from where the entry originated
        #can consider it like a primary tag
        #originated in SAEntry
        #context = None
        #2008.11.06 14:21
        #this seems like the right place to store the source file location
        #then when loading multiple sources into a journal
        #can make changes to an entry and make sure it gets back to the source
        self.source_file = None

    #could also check sub_entries to see how much time was accounted for
    #see if it gets close
    def total_time(self):
        return self.closed_at - self.created_at

    def is_placeholder(self):
        return self.placeholder

    def to_placeholder(self, destination_log):
        if not self.placeholder:
            data = u" [moved / exported to: %s]\n\n" % destination_log
            new = Entry(data, self.created_at, self.closed_at, [destination_log], True)
            return new
        else:
            return self

    def tag_links(self):
        """
        return a list of tuples for each tag's association
        with all other tags
        """
        links = []
        if len(self.tags) > 1:
            for i in range(len(self.tags)-1):
                for j in range(i+1, len(self.tags)):
                    if self.tags[i] != self.tags[j]:
                        tmplist = [self.tags[i], self.tags[i+1]]
                        tmplist.sort()
                        link = tuple(tmplist)
                        if not link in links:
                            links.append(link)

        return links

    def omit_tags(self, tags):
        """
        remove tags from current set of tags if they exist
        scan entries.tags for each omit_tag
        remove it if found
        """
        if tags:
            for omit_tag in tags:
                if omit_tag in self.tags:
                    self.tags.remove(omit_tag)

    def change_newlines(self):
        """
        go through and convert all \r\n to \n
        """
        filtered_data = ''
        for line in self.data.splitlines():
            #[ line ] = multi_filter( [line], updates)
            #[ line ] = multi_filter( [line], path_updates)
            filtered_data += line + '\n'

        self.data = filtered_data

    def render_date(self, date):
        # *2009.02.09 19:15:35 
        # would be better just to use the timestamp directly
        #
        # *2009.03.03 13:03:23 
        # that is trickier in templates when you only have an entry object
        # as is the case in pose/templates/journal/log.mako
        # leaving around for that case
        # (all others removed)
        return str(Timestamp(time=date))

    def render_compact(self):
        ts = Timestamp(time=self.created_at)
        return ts.compact()

    def render_first_line(self):
        """
        since tags may be omitted before exporting a log,
        might not want this to be part of the entry itself
        dunno.. maybe makes more sense there... move omit tag functionality too
        """
        text_time = str(Timestamp(time=self.created_at))
        
        if self.is_placeholder():
            line = '-'
        else:
            line = '*'
        line += text_time + " " + ' '.join(self.tags)
        line += "\n"
        return unicode(line)

    def render_data(self):
        if self.data:
            #make sure that data is buffered with a blank line at the end
            #makes the resulting log easier to read.
            #if there are more than one blanklines, can leave them
            last_line = self.data.splitlines()[-1]
            #not re.match('\s', last_line) and
            
            #are there characters in the last line?  need to adjust if so:
            if re.search('\S', last_line):
                if re.search('\n$', last_line):
                    self.data += "\n"
                else:
                    self.data += "\n"
                    #web entries added will end up with 3 newlines somehow
                    #self.data += "\n\n"
                   
            return unicode(self.data)
        else:
            print "no data in this entry! : %s" % entry
            return ''

    def render(self):
        entry = u''
        entry += self.render_first_line()
        entry += self.render_data()
        return entry

