import StringIO, re, os, sys
from datetime import datetime
import codecs

#from entry import Entry as PythonEntry
#from entry_elixir import Entry as ElixirEntry
#from entry_elixir import Tag, acts_as_taggable, session
from entry import *

from timestamp import *
from tags import Tags

class Log(StringIO.StringIO):
    """
    this defines the file format
    goal is to be easy to create and update in a text editor

    this essentially a text format for a list of entries

    entry index???
    """
    #def read_file(self, filename):
    def __init__(self, filename=None):
        StringIO.StringIO.__init__(self)
        if filename:
            self.log_name = filename
        else:
            self.log_name = ''

        self.seek(0)

    def from_file(self, filename=None):
        #we don't have a file associated with the log:
        if not self.log_name:
            if not filename:
                print "UNKNOWN FILE!"
                exit
            else:
                self.log_name = filename
                
        elif filename and filename != self.log_name:
            print "different file than what log was initialized with"
            exit
            
        else:
            #we have a log_name and no filename... 
            #or the original filename equals the one passed in
            #should be good to go
            pass

        #print self.log_name
        if os.path.exists(self.log_name):
            #f = codecs.open(self.log_name, 'r')
            #f = codecs.open(self.log_name, encoding='utf-8')
            f = codecs.open(self.log_name, encoding='latin_1')
            #f = open(self.log_name, "U")
            self.write(f.read())
            f.close

            #should check to make sure that the file is an actual MomentLog
            #otherwise will have multiple entries for now if multiple
            #text files are imported to the journal

            self.seek(0)

            #not sure that the following does any valid checking
            #especially since we need to allow empty files to be initialized
            #as logs so they can be added to.
            ## if has_timestamp(self.readline()):
            ##     #cool... we're good to go
            ##     self.seek(0)
            ## else:
            ##     #check now happens in to_entries
            ##     pass
            ##     #print "NOT A REAL LOG: %s" % self.log_name

            ##     #this causes problems when creating a new log from scratch
            ##     #pushing check to to_entries function
            ##     #clear it out
            ##     #self.flush()
            ##     #self.close()
            ##     #self = StringIO.StringIO()

            #for debugging:
            #if following is enabled, need to reset self for future read calls
            #print self.read()
            #self.seek(0)
        else:
            print "NO FILE ASSOCIATED WITH LOG: %s" % self.log_name

    def to_file(self):
        if self.log_name:
            f = codecs.open(self.log_name, 'w', encoding='utf-8')
            #f = open(self.log_name, 'w')
            self.seek(0)
            f.write(self.read())
            f.close()
        else:
            print "No log_name for this log"

    def from_entries(self, entries, omits=[]):
        """
        take a collection of entries and put together a log buffer

        #omit tags allow us to omit a certain tag on export/extract

        #assume you're working in a specific mode...
        #don't want to have to write the tag for every entry
        #this allows it to be left out on the extraction

        if omit tag is given,
        do not print the tag for any of the entries.

        import uses add_tags to allow us to assume all entries have that tag
        add_tags allows us to re-add missing tags that were omitted

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        entry should not be an active part of the journal here!!!
        assumes we can change entry permanently
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        """
        #need to make an explicit decision on where to start writing
        #with out that it is completely dependent on where the last
        #seek operation left the pointer in the file
        #could be the beginning, could be the end
        #need to be explicit about how from_entries operates though

        #think the right idea is that from_entries should over write any
        #other data in the log.  that means it should be extracted first
        #if it should be kept
        self.seek(0)
        
        
        last_entry_time = None

        for entry in entries:
            if entry.closed_at:
                #might want to comment this out
                #would lose any manually created end tags
                #this might not be a big deal considering the number of duds
                #it creates
                if entry.closed_at == last_entry_time:
                    #don't need to write the "end" timestamp
                    pass

                #*2008.08.28 07:30
                #only add/import end tags if they already exist...
                #do not attempt to add them automatically on import otherwise
                elif last_entry_time:
                    self.write("*%s end\n" % entry.closed_at.strftime("%Y.%m.%d %H:%M"))

            entry.omit_tags(omits)

            self.write(entry.render())
            #now that working with the entry time itself,
            #need to make sure last_entry_time is a datetime type
            #last_entry_time = self.get_timestamp(entry.data)
            last_entry_time = entry.created_at


    def to_entries(self, add_tags=[]):
        """
        convert log to a list of entry objects (essentially what a log is)
        """
        entries = []

        ph_regex = "^-"
        is_placeholder = re.compile(ph_regex)

        cur_entry = Entry()
        #was calling this context:
        #cur_entry.context = self.log_name
        #now using:
        cur_entry.source_file = self.log_name

        try:
            self.seek(0)
            line = self.readline()
            line = unicode(line)
        except:
            return entries

        #first line of a log should have a timestamp... this is our check
        #if not, don't scan
        if has_timestamp(line):
            while line:
                #look for a timestamp
                if has_timestamp(line):
                    #we found a new entry...

                    #finish up last entry

                    #only need to add if it had information
                    if cur_entry.data:
                        entries.append(cur_entry)

                    new_entry = Entry()
                    new_entry.source_file = self.log_name

                    #need to do something special
                    #something needs to get pushed on a stack,
                    #but not sure exactly how that will work
                    #if current_tags == ["end"]:
                        #print "At the end of the line"
                    #    pass

                    #tempted to just comment this out
                    #not set a closed_at
                    # would be good to do this if there was an explicit
                    # end tag in the previous entry
                    if cur_entry.tags == ["end"]:
                        new_entry.closed_at = cur_entry.created_at

                    (ts, line_tags) = parse_line_for_time(line)
                    new_entry.created_at = Timestamp(ts).time

                    if is_placeholder.match(line):
                        new_entry.placeholder = True

                    #split line and get tags:
                    #assumes timestamp of fixed length. not necessarily the case
                    #line_tags = line[18:-1]
                    current_tags = line_tags.strip().split()
                    if add_tags:
                        #going to reverse the way tags are managed here:
                        #should keep order more logical/hierarchical
                        #for t in add_tags:
                        #    if t not in current_tags:
                        #        current_tags.insert(0, t)

                        temp_tags = add_tags[:]
                        for t in current_tags:
                            if t not in temp_tags:
                                temp_tags.append(t)
                        current_tags = temp_tags

                    new_entry.tags.extend(current_tags)
                    cur_entry = new_entry

                else:
                    # only want to add the entry itself
                    cur_entry.data += line

                line = unicode(self.readline())
                
            #need to get the last entry from the file, if there is one.
            if cur_entry.data:
                entries.append(cur_entry)

        return entries


