import re

def to_tag(item):
    """
    take any string and convert it to an acceptable tag

    tags should not contain spaces or special characters
    numbers, lowercase letters only
    underscores can be used, but they will be converted to spaces in some cases
    """
    item = item.lower()
    item = re.sub(' ', '_', item)
    #todo:
    # filter any non alphanumeric characters
    
    return item

class Tags(list):
    """
    tags are typically just stored as a list of strings
    individual tags should not have spaces
    or non-alphanumeric characters in them (+ the underscore '_')

    this class collects methods
    to help convert to and from
    different formats of expressing tag collections
    """
    def __init__(self, tags=[]):
        #self = []
        list.__init__(self)
        self.extend(tags)
        
    ## def __repr__(self):
    ##     """
    ##     returns just a space separated list of tags
    ##     same format used in an moment log entry
    ##     """
    ##     return ' '.join(self)

    #used to be tags_to_string(tag_string):
    def __str__(self):
        """
        returns just a space separated list of tags
        same format used in an moment log entry
        """
        return ' '.join(self)


    def from_spaced_string(self, tag_string):
        self.extend(tag_string.split(' '))
        return self

    def to_tag_string(self):
        """
        take a list of tags, and return the corresponding tag string
        tag1-tag2
        """
        return '-'.join(self)
        
    #def tags_from_string(tag_string):
    def from_tag_string(self, tag_string):
        #self = tag_string.split('-')
        self.extend(tag_string.split('-'))
        return self
    





def from_tag(item):
    """
    take any tag and attempt to make it more human readable
    doesn't work well in practice

    should just keep them as tags usually
    
    """
    item = re.sub('_', ' ', item)
    parts = item.split(' ')
    new_parts = []
    for p in parts:
        new_parts.append(p.capitalize())
    item = ' '.join(new_parts)
    return item

# This tag functionality has been abstracted to Association
## class Tag:
##     """
##     Object to hold a specific tag
##     """
##     def __init__(self, tag):
##         self.name = tag
##         self.entry_times = []

##     def has_entry(self, entry):
##         #text_time = time_to_text(entry.created_at)
##         text_time = entry.created_at.strftime("%Y.%m.%d %H:%M")
##         if text_time in self.entry_times:
##             return True
##         else:
##             return False

##     def associate_entry(self, entry):
##         if not self.has_entry(entry):
##             text_time = entry.created_at.strftime("%Y.%m.%d %H:%M")
##             #text_time = time_to_text(entry.created_at)
##             self.entry_times.append(text_time)
##             self.entry_times.sort()

##     def remove_entry(self, entry):
##         if self.has_entry(entry):
##             text_time = entry.created_at.strftime("%Y.%m.%d %H:%M")
##             #text_time = time_to_text(entry.created_at)
##             self.entry_times.remove(text_time)


