from datetime import datetime, timedelta
from time import strptime
import re

time_format = "%Y.%m.%d %H:%M:%S"

def has_timestamp(line):
    (ts, remainder) = parse_line_for_time(line)
    if ts:
        return True
    else:
        return False

def get_timestamp(line):
    #original way:
    #removes the leading "*"
    #return line[1:17]
    
    ts, tags = parse_line_for_time(line)
    return ts

def parse_line_for_time(line):
    """
    look at the line, determine if there is a timestamp
    return the timestamp, and the remainder if so
    """
    #most specific
    second_regex = "[-\*](19|20)\d\d[- /.](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01]) \d\d:\d\d:\d\d"
    second_search = re.compile(second_regex)

    minute_regex = "[-\*](19|20)\d\d[- /.](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01]) \d\d:\d\d"
    #time_search = "\*\d\d\d\d.\d\d.\d\d \d\d:\d\d"
    minute_search = re.compile(minute_regex)

    #hopefully optionally match a leading '-' or '*'
    #day_regex = "[-\*]*(19|20)\d\d[- /.](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01])"
    #*2009.03.03 14:38:58
    #requiring the leading '*' now...
    #old logs should be converted manually if not updated at this point
    day_regex = "[-\*](19|20)\d\d[- /.](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01])"
    day_search = re.compile(day_regex)

    s = second_search.match(line)
    m = minute_search.match(line)
    d = day_search.match(line)
    #print s.span()
    if s:
        ts = line[s.span()[0]:s.span()[1]] #s.expand()
    elif m:
        ts = line[m.span()[0]:m.span()[1]] #s.expand()
        #ts = m.expand()
    elif d:
        ts = line[d.span()[0]:d.span()[1]] #s.expand()
        #ts = d.expand()
    else:
        ts = None

    if ts:
        remainder = line[len(ts):]
    else:
        remainder = None

    return [ts, remainder]

class Timestamp(datetime):
    """
    Timestamps have different ways of being formatted
    this object is a common place to store these
    """
    ## #def __init__(self, tstamp=None, time=None, now=False):
    ## def __init__(self, tstamp=None, cstamp=None):
    ##     if tstamp:
    ##         self.from_text(tstamp)
    ##     elif cstamp:
    ##         self.from_compact(cstamp)
    ##     #else:
    ##     #    datetime.__init__(self, *args, **kw)
            
    ##     ## elif now:
    ##     ##     self = datetime.now()
    ##     ## else:
    ##     ##     self = time

    def __str__(self):
        text_time = ''
        if self.strftime("%S") != "00":
            text_time = self.strftime("%Y.%m.%d %H:%M:%S")
        elif self.strftime("%H:%M") != "00:00":
            text_time = self.strftime("%Y.%m.%d %H:%M")
        else:
            text_time = self.strftime("%Y.%m.%d")
        return text_time

    ## def now(self):
    ##     self = datetime.now()

    #was time_to_tstamp
    def compact(self):
        """
        take a python datetime object
        return a string representation of that time
        YYYYMMDDHHMMSS

        very similar to moments.entry.render_date
        """
        if (self.hour == 0) and (self.minute == 0) and (self.second == 0):
            return self.strftime("%Y%m%d")
        elif (self.second == 0):
            return self.strftime("%Y%m%d%H%M")
        else:
            return self.strftime("%Y%m%d%H%M%S")

    #was tstamp_to_time
    def from_compact(self, text_time):
        """
        take a string of the format:
        YYYYMMDDHHMMSS
        return a python datetime object
        """
        if len(text_time) == 14:
            self = datetime(*(strptime(text_time, "%Y%m%d%H%M%S")[0:6]))
        elif len(text_time) == 12:
            self = datetime(*(strptime(text_time, "%Y%m%d%H%M")[0:6]))
        elif len(text_time) == 8:
            self = datetime(*(strptime(text_time, "%Y%m%d")[0:6]))
        elif len(text_time) == 6:
            self = datetime(*(strptime(text_time, "%Y%m")[0:6]))
        elif len(text_time) == 4:
            self = datetime(*(strptime(text_time, "%Y")[0:6]))
        else:
            #some other format
            self = None
        return self

    #was log.text_to_time
    def from_text(self, text_time):
        """
        take a string of the format:
        YYYY.MM.DD HH:MM
        return a python datetime object
        """
        #removed leading char if needed
        if re.match("[-\*]", text_time):
            text_time = text_time[1:]

        if len(text_time) == 19:
            # this only works with python 2.5
            # (current default is 2.4.4 for zope)
            #return datetime.strptime(text_time, self.text_time_format)
            time = datetime(*(strptime(text_time, "%Y.%m.%d %H:%M:%S")[0:6]))
        elif len(text_time) == 16:
            time = datetime(*(strptime(text_time, "%Y.%m.%d %H:%M")[0:6]))
        elif len(text_time) == 10:
            try:
                time = datetime(*(strptime(text_time, "%Y.%m.%d")[0:6]))
            except:
                try:
                    time = datetime(*(strptime(text_time, "%Y-%m-%d")[0:6]))
                except:
                    time = datetime(*(strptime(text_time, "%Y/%m/%d")[0:6]))
        else:
            #some other format
            time = None

        self = time
        return time
    
class Timerange:
    """
    Timerange holds a start and end python datetime

    trange can either be a string representing two timestamps separated
    by a '-'
    or just a simple tstamp string
    in which case it will be evaluated as a range based on accuracy
    """
    def __init__(self, trange=None, start=None, end=None):
        if trange:
            self.from_trange(trange)
        else:
            self.start = start
            self.end = end

    def as_tuple(self):
        return (self.start, self.end)

    def from_trange(self, trange, end_is_now=False):
        """
        will work with either a simple timestamp string
        or a string with a - separating two timestamp strings
        
        check the tstamp for a range of times
        split if found
        return the range start and end

        end defaults to now or tstamp_to_timerange end
        """
        end = ''
        start = ''
        if re.search('-', trange):
            (start, end) = trange.split('-')
        else:
            start = trange

        (start, default_end) = self.from_tstamp(start)

        if end:
            ts = Timestamp().from_compact(end)
            end = ts.time
            #end = tstamp_to_time(end)
        elif end_is_now:
            end = datetime.now()
        else:
            end = default_end

        self.end = end
        self.start = start
        return (start, end)

    #was tstamp_to_timerange
    def from_tstamp(self, text_time):
        """
        take a string of the format:
        YYYYMMDDHHMMSS
        return a tuple of python datetime objects
        based on the format supplied
        can assume different ranges based on the degree of accuracy

        we want to assume the difference 
        """
        if len(text_time) == 14:
            start = datetime(*(strptime(text_time, "%Y%m%d%H%M%S")[0:6]))
            end = start

        elif len(text_time) == 12:
            start = datetime(*(strptime(text_time, "%Y%m%d%H%M")[0:6]))
            delta = timedelta(minutes=1)
            end = start+delta

        elif len(text_time) == 10:
            start = datetime(*(strptime(text_time, "%Y%m%d%H")[0:6]))
            delta = timedelta(hours=1)
            end = start+delta

        elif len(text_time) == 8:
            start = datetime(*(strptime(text_time, "%Y%m%d")[0:6]))
            delta = timedelta(days=1)
            end = start+delta

        elif len(text_time) == 6:
            start = datetime(*(strptime(text_time, "%Y%m")[0:6]))
            month = start.month
            if month == 12:
                month = 1
            else:
                month += 1
                
            #there is no concept of months in timedelta:
            #delta = timedelta(months=1)
            #end = start+delta
            end = datetime(start.year, month, 1)

        elif len(text_time) == 4:
            start = datetime(*(strptime(text_time, "%Y")[0:6]))
            delta = timedelta(days=365)
            end = start+delta

        else:
            #some other format
            start = None
            end = None
        
        self.start = start
        self.end = end
        return (start, end)
