"""Routes configuration

The more specific and detailed routes should be defined first so they
may take precedent over the more generic routes. For more information
refer to the routes manual at http://routes.groovie.org/docs/
"""
from pylons import config
from routes import Mapper

def make_map():
    """Create, configure and return the routes Mapper"""
    map = Mapper(directory=config['pylons.paths']['controllers'],
                 always_scan=config['debug'])
    map.minimization = False
    
    # The ErrorController route (handles 404/500 error pages); it should
    # likely stay at the top, ensuring it can always be resolved
    map.connect('/error/{action}', controller='error')
    map.connect('/error/{action}/{id}', controller='error')

    # CUSTOM ROUTES HERE

    map.connect('/tester/{action}', controller='tester')

    map.connect('/admin/import_usb/{content}', controller='admin', action='import_usb')
    #map.connect('/admin/import_audio', controller='admin', action='import_audio')
    map.connect('/admin/export_logs', controller='admin', action='export_logs')
    map.connect('/admin/index', controller='admin', action='index')

    map.connect('/image/update', controller='image', action='update')
    map.connect('/image/full/*(dname)/{iname}', controller='image', action='full')
    map.connect('/image/*(dname)/{iname}', controller='image', action='view')

    map.connect('/sound/*(dname)/{fname}', controller='sound', action='play')
    
    map.connect('/playlist/file/*(fpath)', controller='playlist', action='file', format='file')
    map.connect('/playlist/xspf/*(fpath)', controller='playlist', action='file', format='xspf')
    map.connect('/playlist/m3u/*(fpath)', controller='playlist', action='file', format='m3u')
    map.connect('/playlist/edit/*(fpath)', controller='playlist', action='edit')
    map.connect('/playlist/update/*(fpath)', controller='playlist', action='update')
    map.connect('/playlist/export/*(fpath)', controller='playlist', action='export')
    map.connect('/playlist/*(fpath)', controller='playlist', action='index')


    map.connect('/moment/update/{tstamp}/*(fpath)', controller='moment', action='update')
    map.connect('/moment/update/{tstamp}/', controller='moment', action='update')
    map.connect('/moment/update/{tstamp}', controller='moment', action='update')
    #could be creating new, and passing tstamp over request.params
    #map.connect('/moment/update/*(fpath)', controller='moment', action='update')
    #map.connect('/moment/update', controller='moment', action='update')
    map.connect('/moment/delete/{tstamp}/*(fpath)', controller='moment', action='delete')
    map.connect('/moment/delete/{tstamp}', controller='moment', action='delete')
    map.connect('/moment/edit/{tstamp}/*(fpath)', controller='moment', action='edit')
    map.connect('/moment/edit/{tstamp}', controller='moment', action='edit')
    #if want to keep consistency with /moment/edit
    #could check to see if it is an actual timestamp in the route
    #otherwise creating a new moment in a specific fpath
    #would try to be interpreted as timestamp
    map.connect('/moment/new/*(fpath)', controller='moment', action='edit')
    map.connect('/moment/new', controller='moment', action='edit')
    map.connect('/moment/{tstamp}/*(fpath)', controller='moment', action='index')
    map.connect('/moment/{tstamp}', controller='moment', action='index')
    map.connect('/moment/', controller='moment', action='create')


    map.connect('/tag/cloud/*(fpath)', controller='tag', action='cloud')
    map.connect('/tag/cloud/', controller='tag', action='cloud', fpath='')
    map.connect('/tag/extract/{tag_string}/*(fpath)', controller='tag', action='extract')
    map.connect('/tag/extract/{tag_string}/', controller='tag', action='extract', fpath='')
    map.connect('/tag/extract/{tag_string}', controller='tag', action='extract', fpath='')
    map.connect('/tag/intersect/{tag_string}/*(fpath)', controller='tag', action='intersect')
    map.connect('/tag/intersect/{tag_string}/', controller='tag', action='intersect', fpath='')
    map.connect('/tag/intersect/{tag_string}', controller='tag', action='intersect', fpath='')
    map.connect('/tag/{tag_string}/*(fpath)', controller='tag', action='index')
    map.connect('/tag/{tag_string}/', controller='tag', action='index', fpath='')
    map.connect('/tag/{tag_string}', controller='tag', action='index', fpath='')


    map.connect('/time/{trange}/*(fpath)', controller='journal', action='time')
    map.connect('/time/{trange}/', controller='journal', action='time', fpath='')
    map.connect('/time/{trange}', controller='journal', action='time', fpath='')

    map.connect('/journal/entry_edit/*(fpath)', controller='journal', action='entry_edit')
    map.connect('/journal/entry_delete/*(fpath)', controller='journal', action='entry_delete')
    map.connect('/journal/edit/*(fpath)', controller='journal', action='edit')
    map.connect('/journal/update/*(fpath)', controller='journal', action='update')
    map.connect('/journal/images/dupes/*(fpath)', controller='journal', action='images', dupes='show')
    map.connect('/journal/images/save/*(fpath)', controller='journal', action='images', save=True)
    map.connect('/journal/images/*(fpath)', controller='journal', action='images')
    map.connect('/journal/sounds/dupes/*(fpath)', controller='journal', action='sounds', dupes='show')
    map.connect('/journal/sounds/save/*(fpath)', controller='journal', action='sounds', save=True)
    map.connect('/journal/sounds/*(fpath)', controller='journal', action='sounds')
    map.connect('/journal/local_links/dupes/*(fpath)', controller='journal', action='local_links', dupes='show')
    map.connect('/journal/local_links/save/*(fpath)', controller='journal', action='local_links', save=True)
    map.connect('/journal/local_links/*(fpath)', controller='journal', action='local_links')
    map.connect('/journal/local_links2/*(fpath)', controller='journal', action='local_links2')
    map.connect('/journal/links/dupes/*(fpath)', controller='journal', action='links', dupes='show')
    map.connect('/journal/links/*(fpath)', controller='journal', action='links')

    map.connect('/journal/log/*(fpath)', controller='journal', action='log')
    map.connect('/journal/*(fpath)', controller='journal', action='index')
    map.connect('/journal/', controller='journal', action='index', fpath='')
    map.connect('/journal', controller='journal', action='index', fpath='')

    map.connect('/html/edit/*(fpath)', controller='html', action='edit')
    map.connect('/html/update/*(fpath)', controller='html', action='update')
    map.connect('/html/*(fpath)', controller='html', action='index')
    map.connect('/html/', controller='html', action='index', fpath='')
    map.connect('/html', controller='html', action='index', fpath='')

    #would be nice to merge file, media, and text concepts if possible
    #seems fragmented
    map.connect('/text/edit/*(fpath)', controller='text', action='edit')
    map.connect('/text/update/*(fpath)', controller='text', action='update')
    map.connect('/file/mplayer/*(dname)/{mname}', controller='media', action='mplayer')
    map.connect('/file/tag/*(dname)/{fname}', controller='media', action='tag')
    map.connect('/file/*(dname)/{fname}', controller='media', action='index')


    #map.connect('/dir/create/*(dname)', controller='directory', action='create')
    map.connect('/dir/create/', controller='directory', action='create')
    map.connect('/dir/create', controller='directory', action='create')
    map.connect('/dir/update_many', controller='directory', action='update_many')
    map.connect('/dir/as_links/*dname', controller='directory', action='as_links')
    map.connect('/dir/auto_rotate/*dname', controller='directory', action='auto_rotate')
    map.connect('/dir/slideshow/*dname', controller='directory', action='slideshow')
    
    map.connect('/dir/*dname', controller='directory', action='directory')
    map.connect('/dir/', controller='directory', action='index')
    map.connect('/dir', controller='directory', action='index')
    #map.connect('/*dname', controller='directory', action='directory')
    map.connect('/', controller='directory', action='index')

    #map.connect('/{controller}/{action}')
    #map.connect('/{controller}/{action}/{id}')

    return map
