import logging

from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from pose.lib.base import BaseController, render
from pose import model

log = logging.getLogger(__name__)

import os, subprocess

from pylons import config
image_usb_path = config['image_usb_path']
image_local_path = config['image_local_path']
audio_usb_path = config['audio_usb_path']
audio_local_path = config['audio_local_path']

#admin = config['admin']
local_path = config['local_path']

from medialist.medialist import MediaList

class AdminController(BaseController):

    def index(self):
        # Return a rendered template
        #   return render('/template.mako')
        # or, Return a response
        return 'Hello World'

    def export_logs(self):
        """
        2008.11.14 15:54
        should look at the config['log_usb_path']
        see if any of the files in config['log_local_path']
        are in the usb path
        if so,
        create a journal instance
        merge the two
        export to the usp path
        remove the local instance
        2008.11.19 12:20:03
        
        """
        result = ''
        local_logs = model.make_node(config['log_local_path'])
        local_logs.scan_filetypes()

        try:
            add_tags = [ "system:%s" % config['server_name'] ]
            result += "ADDING SERVER TAG: %s<br>" % (str(add_tags))
        except:
            #can uncomment following 2 lines if don't want server tag:
            #result += "NO SERVER NAME in config<br>"
            #add_tags = []
            return "NO SERVER NAME in config<br>"

        try:
            usb_logs = model.make_node(config['log_usb_path'])
        except:
            return "NO USB PATH FOUND.  Is it plugged in?"
        usb_logs.scan_filetypes()

        usb_log_paths = MediaList(usb_logs.logs, item_type="File")
        

        result += 'LOCALS:<br>'
        #for f in local_logs.logs.as_objects():
        for f in local_logs.logs:
            usb_f = os.path.join(config['log_usb_path'], f.name)
            j = f.create_journal(add_tags)

            if usb_f in usb_log_paths:
                result += "<br>ALSO ON USB: " + f.name + '<br>'
                
                len1 = len(j.to_entries())
                usb_node = model.make_node(usb_f)

                #for testing length only:
                j2 = usb_node.create_journal()
                len2 = len(j2.to_entries())

                j.from_file(usb_f)
                len3 = len(j.to_entries())

                result += "merge resulted in %s entries from %s and %s<br>" % (len3, len1, len2)
                if len1+len2 == len3:
                    result += "SAVING<br>"
                    j.to_file(usb_f)
                    os.remove(f.path)
                    
                else:
                    result += "WARNING: dupes/conflicts encountered<br>"

            else:
                result += "moving: %s to %s<br>" % (f.path, config['log_usb_path'])
                #rather than move, write with machine tag
                #one problem with this is the log timestamp will change
                #but it will also change on subsequent merges too
                #so unless it is retained everywhere, not a big deal
                #(plus timestamps in log and filename should suffice!)
                j.to_file(usb_f)
                os.remove(f.path)
                
                #mv = subprocess.Popen("mv %s %s/" % (f.path, config['log_usb_path']), shell=True, stdout=subprocess.PIPE)
                #mv.wait()
                
        
        result += '<br>USBS:<br>'

        #this returns full path, but it is sorted at least:
        usb_log_paths.sort()
        for f in usb_log_paths:
            result += f + '<br>'
            
##         #this returns items in random order:
##         for f in usb_logs.logs:
##             result += f.name + '<br>'

        return result
    
    def import_usb(self, content="pics"):
        if content == "pics":
            #d = model.Directory(usb_path)
            d = model.make_node(image_usb_path)
            #d.scan_filetypes()
            #images = d.get_images().as_objects()
            #images = d.images
            new_dir = os.path.join(image_local_path, d.file_date_range())

        elif content == "audio":
            d = model.make_node(audio_usb_path)
            new_dir = os.path.join(audio_local_path, d.file_date_range())

        elif content == "vinyl":
            d = model.make_node(audio_usb_path)
            new_dir = os.path.join(config['vinyl_local_path'], d.file_date_range())

        elif content == "covers":
            d = model.make_node(config['sony_usb_path'])
            new_dir = os.path.join(config['covers_local_path'], d.file_date_range())
            
        else:
            result = "UNKNOWN CONTENT TO IMPORT!"
            return result

        #don't forget to return what the import did!
        result = self._move_files(d.path, new_dir)
        return result
                
    def _move_files(self, source_dir, new_dir):
        """
        move all files on camera / usb media
        to local image directory
        """
        ## if content == "audio":

        ## elif content == "pics":
        ##     new_dir = os.path.join(image_local_path, new_name)
        ## else:
        ##     new_dir = os.path.join(local_path, 'incoming', new_name)
            
        if not os.path.isdir(new_dir):
            os.mkdir(new_dir)

        #following yields:
        #OSError: [Errno 18] Cross-device link
        #on Mac OS X
        #may work elsewhere
        #result = ''
        #for item in d.contents:
        #    orig = os.path.join(d.path, item)
        #    new = os.path.join(new_dir, item)
        #    result += "%s %s\n" % (orig, new)
        #    os.rename(orig, new)

        #instead, just issue system command
        mv = subprocess.Popen("mv %s/* %s" % (source_dir, new_dir), shell=True, stdout=subprocess.PIPE)
        mv.wait()
        result = "All finished with moving files.  Any output included below:<br>\n"
        result += mv.stdout.read()
        result += "\n<br> All files moved to: %s" % new_dir
        result += "\n<br> Press Back to return <br> \n"
        
        return result
