import logging

from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from pose.lib.base import BaseController, render
from pose import model

log = logging.getLogger(__name__)

from StringIO import StringIO

from webhelpers.html import literal

from text import TextController
import re, os

from moments.log import Log
from moments.journal import Journal
from moments.timestamp import Timerange

from medialist.medialist import MediaList, multi_filter
from medialist.filters import path_updates

def journal2medialist(journal):
    """
    take a journal,
    generate the entry frequencies,
    return a media list based on that priority

    (assumes 1 item per journal entry?)
    """
    m = MediaList(item_type='nodes', prioritize=True)

    ilist = journal.datas.frequency_list()
    ilist.sort()
    ilist.reverse()
    for i in ilist:
        #could possibly be more than one file listed in an entry,
        #so new lines will be included
        for line in i[1].splitlines():
            try:
                tnode = model.make_node(line)
            except:
                tnode = None
                print "skipping item: %s" % i[1]
            if tnode:
                m.add_item(tnode)
    return m

def journal_frequency_medialist(journal, tags, updates):
    """
    create a journal based on node,
    filter based on tags
    apply updates to all items in all entries
    create a new journal to find the frequency of all items
    create a media list based on that frequency

    this works if every entry contains one and only one file per entry
    entries with mulitple files per entry need special treatment

    """
    #node = model.make_node(fpath)
    #c.node = node

    #better off using osbrowser to generate the log for us, as needed:
    #j = node.create_journal()


    entries = journal.union_tags(tags)
    #entries.reverse()

    #normalize all of the data first...
    #as the system changes, so do the paths
    new_entries = []
    for e in entries:
        #new_data = e.data.replace('\r\n', '\n')
        filtered_data = ''
        for line in e.data.splitlines():
            if line:
                [ line ] = multi_filter( [line], updates)
                #[ line ] = multi_filter( [line], path_updates)
                if line:
                    filtered_data += line + '\n'

        #filtered_data += '\n'
        e.data = filtered_data
        new_entries.append(e)

    #make a new journal with normalized data
    j2 = Journal()
    j2.from_entries(new_entries)

    m = journal2medialist(j2)
    return m
    
class JournalController(TextController):

    def index(self, fpath):
        #${phraseUnicode2ASCII(line)} <br>
        #${line} <br>

        node = model.make_node(fpath)

        j = node.create_journal()
        
        c.entries = j.to_entries()
        c.node = node
        return render('/journal/log.mako')
        
    def edit(self, fpath):
        node = model.make_node(fpath)
        f = file(node.path)
        c.data = literal(f.read())
        c.node = node
        #this is the only difference from text controller's edit
        c.controller = "journal"
        return render('/text-edit.mako')

    def time(self, trange, fpath):
        """
        will depend on the url format of the tstamp
        for now, and for simplicity,
        thinking that it should be fine to just pass both start and end
        in in the same tstamp var as follows:
        start-end

        where start and end are both timestamps in the format:
        YYYYMMDDmmss (and everything after YYYY is optional)
        """
        node = model.make_node(fpath)

        j = node.create_journal()

        (start, end) = Timerange(trange).as_tuple()
        
        c.entries = j.limit(start, end)

        c.node = node
        return render('/journal/log.mako')
        

    def log(self, fpath):
        c.fpath = fpath
        l = model.make_node(fpath)
        c.node = l

        log = Log(l.path)
        log.from_file()
        c.entries = log.to_entries()        

        return render('/journal/log.mako')

    def images(self, fpath, dupes=None, save=False):
        """
        read in a log
        [extract image entries, or assume all entries contain local images]
        show a summary from the resulting playlist
        """
        node = model.make_node(fpath)
        c.node = node

        #this will only work if the fpath is an actual log
        #log = Log(l.path)
        #log.from_file()        
        #entries = log.to_entries()
        #most recent first:
        #entries.reverse()

        #better off using osbrowser to generate the log for us, as needed:
        j = node.create_journal()


        #this gets everything:
        #entries = j.to_entries()

        tags = ["view", "access"]
        updates = [ ('', ''),
                    
                    ]
        
        #print type(dupes)
        if not dupes:
            #print "PRIORITIZING"
            #m = MediaList(item_type='nodes', prioritize=True)
            m = journal_frequency_medialist(j, tags, updates)
        else:
            m = MediaList(item_type='nodes')


            entries = j.union_tags(tags)

            for e in entries:
                #not sure where the \r\n is being introduced
                #doesn't seem to come from the entry initialization
                #cleaning up here for now:
                new_data = e.data.replace('\r\n', '\n')
                for line in new_data.split('\n'):
                    if line:
                        [ line ] = multi_filter( [line], updates)
                        try:
                            tnode = model.make_node(line)
                            m.add_item(tnode)
                        except:
                            print "skipping image: %s" % line


        #m.sort_items("reverse")
        c.items = m

        if save:
            response.headers['Content-Type'] = "audio/x-mpegurl"
            response.charset = 'utf8'
            return m.to_m3u()
        else:
            return render('/journal/as_images.mako')

    def sounds(self, fpath, dupes=None, save=False):
        """
        read in a log
        [extract image entries, or assume all entries contain local images]
        show a summary from the resulting playlist
        """
        node = model.make_node(fpath)
        c.node = node

        #better off using osbrowser to generate the log for us, as needed:
        j = node.create_journal()

        #filters local/specific to this function
        updates = [ ('', ''),
                    ]
        updates.extend(path_updates)

        m = journal_frequency_medialist(j, ['sound'], updates)

        #m.sort_items("reverse")
        c.sounds = m.as_objects()
        
        if save:
            response.headers['Content-Type'] = "audio/x-mpegurl"
            response.charset = 'utf8'
            return m.to_m3u()
        else:
            return render('/playlist.mako')


    def local_links(self, fpath, save=False):
        """
        read in a log
        [extract image entries, or assume all entries contain local images]
        show a summary from the resulting playlist
        """
        node = model.make_node(fpath)
        c.node = node

        #better off using osbrowser to generate the log for us, as needed:
        j = node.create_journal()

        tags = []
        entries = j.union_tags(tags)
        
        m = MediaList()

        playlists = []
        
        for e in entries:
            temp_m = MediaList()
            temp_m.from_copy_all_urls(e.data)
            updates = [ 
                        ('http:\/\/localhost:5000\/', ''),
                        ('http:\/\/localhost:5001\/', ''),
                        ('http:\/\/localhost:5002\/', ''),
                        ('http:\/\/localhost:5003\/', ''),
                        ('image\/', ''),
                        ('dir\/', ''),
                        ('00pictures', 'graphics'),
                        ]

            temp_m.multi_filter(updates)
            
            temp_m.from_relative()

            for i in temp_m:
                m.add_item(i)
                

        new_m = m.flatten_and_sort()
        #new_m = m


        playlists.append(new_m)
            
        c.p = playlists
        
        if save:
            response.headers['Content-Type'] = "audio/x-mpegurl"
            response.charset = 'utf8'
            return new_m.to_links()
        else:
            return render('/journal/log_list.mako')


    def local_links2(self, fpath, dupes=None):
        """
        read in a log
        [extract image entries, or assume all entries contain local images]
        show a summary from the resulting playlist
        """
        l = model.make_node(fpath)
        c.node = l

        log = Log(l.path)
        log.from_file()
        
        #c.log = l.log
        entries = log.to_entries()

        ## print type(dupes)
        if not dupes:
            print "PRIORITIZING"
            m = MediaList(prioritize=True)
            entries.reverse()
        else:
            m = MediaList()

        #m = MediaList(item_type='nodes', prioritize=True)

        playlists = []
        
        for e in entries:
            temp_m = MediaList()
            temp_m.from_copy_all_urls(e.data)
            updates = [ ('http:\/\/localhost:5002\/', ''),
                        ('image\/', ''),
                        ('dir\/', ''),
                        ('00pictures', 'graphics'),
                        ]

            temp_m.multi_filter(updates)
            
            ## m.replace('http:\/\/localhost:5002\/', '')
            ## m.replace('image\/', '')
            ## m.replace('dir\/', '')
            ## m.replace('00pictures', 'graphics')

            temp_m.from_relative()

            if dupes:
                playlists.append(temp_m)
            else:
                for i in temp_m:
                    m.add_item(i)
                

        ## for e in entries:
        ##     for line in e.data.split('\n'):
        ##         if line:
        ##             tnode = model.make_node(line)
        ##             m.add_item(tnode)
        ##             #m.append(str(tnode))
        ##             #m.items[str(tnode)] = tnode

        if not dupes:
            playlists.append(m)
            
        c.p = playlists
        
        return render('/journal/log_list.mako')

        
    def local_links3(self, fpath, dupes=None):
        """
        read in a log
        [extract image entries, or assume all entries contain local images]
        show a summary from the resulting playlist
        """
        node = model.make_node(fpath)
        c.node = node

        #better off using osbrowser to generate the log for us, as needed:
        j = node.create_journal()

        updates = [ #('((?!^http)|(?!^\/))*', ''),
                    ('http:\/\/localhost:5002\/', ''),
                    ('image\/', ''),
                    ('dir\/', ''),
                    ('00pictures', 'graphics'),
                    ]
        #updates.extend(path_updates)

        m = journal_frequency_medialist(j, ['sound'], updates)

        m = journal2medialist(j2)

        playlists = []
        playlists.append(m)
        c.p = playlists
        print str(j.to_entries())
        return render('/journal/log_list.mako')
