import logging

from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from pose.lib.base import BaseController, render

log = logging.getLogger(__name__)

#from pose import model
from osbrowser.meta import make_node

#from journal import tstamp_to_timerange, time_to_tstamp
from moments.timestamp import Timerange, Timestamp
from moments.tags import Tags
from moments.entry import Entry

from routes import url_for

import formencode
from formencode import htmlfill

class EntryForm(formencode.Schema):
    allow_extra_fields = True
    filter_extra_fields = True
    email = formencode.validators.Email(not_empty=True)
    date = formencode.validators.DateConverter(not_empty=True)


class MomentController(BaseController):

    def index(self, tstamp=None, fpath=''):
        if tstamp:
            (start, end) = Timerange(tstamp).as_tuple()
            #(start, end) = tstamp_to_timerange(tstamp)
            
            node = make_node(fpath)
            c.node = node

            j = node.create_journal()

            entries = j.limit(start, end)
            if len(entries) < 1:
                return "No entries found then and there"
            elif len(entries) > 1:
                return "More than one entry found... please narrow"
            else:
                c.entry = entries[0]
                return render('/moment-view.mako')

        else:
            return render('/moment-edit.mako')

    def edit(self, tstamp=None, fpath=''):
        #c.rendered = time_to_tstamp()
        #print "Tstamp: %s, fpath: %s" % (tstamp, fpath)
        

        node = make_node(fpath)
        c.node = node
        if tstamp:
            j = node.create_journal()

            (start, end) = Timerange(tstamp).as_tuple()
            #(start, end) = tstamp_to_timerange(tstamp)
            c.entries = j.limit(start, end)
            
            if len(c.entries) < 1:
                return "No entries found then and there"
            elif len(c.entries) > 1:
                return "More than one entry found... please narrow"
            else:
                #for k, v in c.entries[0].__dict__.items():
                #    print "%s is %s" % (k, v)

                c.timestamp = Timestamp(time=c.entries[0].created_at)
                #print str(c.timestamp)
                
                #need to use htmlfill here
                return htmlfill.render(render('/moment-edit.mako'), c.entries[0].__dict__)

        else:
            #c.now = Timestamp(now=True)
            c.timestamp = Timestamp(now=True)
            #c.rendered = c.now.compact()

            return render('/moment-edit.mako')

    def update(self, tstamp, fpath=''):

        if not fpath:
            fpath = "journal.txt"
        node = make_node(fpath)
            
        #always need a timestamp, otherwise wouldn't know what moment to edit
        (start, end) = Timerange(tstamp).as_tuple()

        #need to find the actual entry's file that we're updating
        #need to find the entry first then
        #(would be nice to redirect in edit when finding entry there)
        if node.find_type() == "Directory":
            j = node.create_journal()
            entries = j.limit(start, end)
            if len(entries) == 1:
                node = make_node(entries[0].source_file, relative=False)

        #at this point, node should be a file:
        j = node.create_journal()
        entries = j.limit(start, end)
        if len(entries) > 1:
            return "Too many entries for an update"
        elif len(entries) == 0:
            #must have a new entry:
            entry = Entry()
            entry.created_at = start
        else:
            entry = entries[0]

        #since returning if improper number of entries found, safe to continue

        #sync entry with submit results
        #could use any form library here
        entry.data = request.params['data']

        #data from browser has \r\n
        entry.change_newlines()

        #web seems to add extra newlines... this should zap it
        #entry.data = ''.join(entry.data.splitlines()[:-1])
        #print entry.data
        
        entry.tags = Tags(request.params['tags'].split(' '))
        #could also use:
        #entry.tags = Tags().from_spaced_string(request.params['tags'])

        #####################################################
        # INDICATE AN UPDATE OCCURRED IN ENTRY???
        ######################################################

        if len(entries):
            #update the entry in the journal
            j.replace_entry(entry)
        else:
            j.update_entry(entry, position=0)

        #save the journal file
        j.to_file()
        ## f = codecs.open(node.path, encoding='utf-8', mode='w+')
        ## f.write(request.params['data'])
        ## f.close()

        session['flash'] = "Saved changes!"
        session.save()

        redirect_to(url_for(controller="journal", action="index", fpath=fpath))


    def delete(self, tstamp, fpath):
        """
        copied from update... should refactor where possible

        i.e. code to find entry and redirect to that file
        """

        node = make_node(fpath)
        #always need a timestamp, otherwise wouldn't know what moment to edit
        (start, end) = Timerange(tstamp).as_tuple()

        #need to find the actual entry's file that we're updating
        #need to find the entry first then
        #(would be nice to redirect in edit when finding entry there)
        if node.find_type() == "Directory":
            j = node.create_journal()
            entries = j.limit(start, end)
            if len(entries) == 1:
                node = make_node(entries[0].source_file, relative=False)

        #at this point, node should be a file:
        j = node.create_journal()
        entries = j.limit(start, end)
        if len(entries) != 1:
            return "Wrong number of entries for an update"
        else:
            entry = entries[0]

            #sync entry with submit results
            #could use any form library here
            entry.data = request.params['data']

            #data from browser has \r\n
            entry.change_newlines()
            
            entry.tags = Tags(request.params['tags'].split(' '))
            #could also use:
            #entry.tags = Tags().from_spaced_string(request.params['tags'])

            #####################################################
            # INDICATE AN UPDATE OCCURRED IN ENTRY???
            ######################################################

            #update the entry in the journal
            j.replace_entry(entry)

            #save the journal file
            j.to_file()
            ## f = codecs.open(node.path, encoding='utf-8', mode='w+')
            ## f.write(request.params['data'])
            ## f.close()

            session['flash'] = "Saved changes!"
            session.save()

            redirect_to(url_for(action="edit"))
