import logging

from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from webhelpers.html import literal

from pose.lib.base import BaseController, render
from pose import model

log = logging.getLogger(__name__)

from text import TextController

from medialist.medialist import MediaList
from medialist.filters import path_updates

from journal import journal_frequency_medialist

class PlaylistController(TextController):

    def index(self, fpath):
        """
        render a playlist that can be played in the browser
        """
        node = model.make_node(fpath)
        #playlist = model.MediaList(node=node)
        playlist = MediaList()
        playlist.from_file(node.path)
        playlist.multi_filter()

        c.sounds = playlist.as_objects()
        c.node = node
        # Return a rendered template
        #   return render('/template.mako')
        # or, Return a response
        #return 'Hello World: %s' % str(playlist)
        return render('/playlist.mako')

    def edit(self, fpath):
        node = model.make_node(fpath)
        f = file(node.path)
        #c.data = literal(model.phraseUnicode2ASCII(f.read()))
        c.data = literal(f.read())
        c.node = node
        c.controller = "playlist"
        return render('/text-edit.mako')


    def export(self, fpath):
        """
        call exportPlaylists function on an iTunes Library node
        """
        node = model.make_node(fpath)
        exports = "Exporting the following playlists from the library:\n<br>"
        exports += node.exportPlaylists()
        return exports
        

    #merging m3u, file and xspf into one... just pass in format
    def file(self, format, fpath, journal=False):
        """
        return a playlist as a file
        """
        node = model.make_node(fpath)
        
        #response.headers['Content-type'] = "audio/x-mpegurl"

        if node.find_type() == "Directory":
            #print "THIS IS A DIRECTORY: %s" % node.path
            node.scan_filetypes()

            #we'll assume that if there are sounds, we just want to show them
            if len(node.sounds):
                playlist = MediaList(node.sounds, item_type="Sound")
                playlist.sort()
                
            #other wise must be a directory that might contain many journal
            #files in sub-directories
            #create a journal from those, and a medialist from that
            else:
                j = node.create_journal()
                playlist = journal_frequency_medialist(j, ['sound'], path_updates)

        # we know we have a single file
        # need to determine if it is a playlist or a jornal
        else:
            if node.find_type() == "Playlist":
                #print "THIS IS A PLAYLIST: %s" % node.path
                playlist = MediaList(source=node.path)
                playlist.from_file()
                playlist.multi_filter()
            else:
                j = node.create_journal()
                playlist = journal_frequency_medialist(j, ['sound'], path_updates)
                

        if format == 'm3u':
            m3u = playlist.to_m3u()
            return m3u 
        elif format == 'xspf':
            return str(playlist.to_xspf())
        else:
            #default as a static file
            response.headers['Content-Type'] = "audio/x-mpegurl"
            response.charset = 'utf8'
            return playlist.to_m3u()

            
    ## def file(self, fpath):
    ##     """
    ##     return a playlist as a file
    ##     """
    ##     node = model.make_node(fpath)
    ##     playlist = MediaList(source=node.path)
    ##     playlist.from_file()
    ##     playlist.multi_filter()

    ## #merge m3u and xspf into one... just pass in format
    ## def m3u(self, fpath):
    ##     """
    ##     return a playlist as a file
    ##     """
    ##     node = model.make_node(fpath)
        
    ##     #response.headers['Content-type'] = "audio/x-mpegurl"

    ##     #if str(node.__class__) == "pose.model.osbrowser.Directory":
    ##     if node.find_type() == "Directory":
    ##         print "THIS IS A DIRECTORY: %s" % node.path
    ##         node.scan_filetypes()
    ##         playlist = MediaList(node.sounds, item_type="Sound")
            
    ##         #return str(playlist.to_xspf())
    ##         return playlist.to_m3u()
    ##     else:
    ##         print "THIS IS A PLAYLIST: %s" % node.path
    ##         playlist = MediaList(source=node.path)
    ##         playlist.from_file()
    ##         playlist.multi_filter()
    ##         m3u = playlist.to_m3u()
    ##         #print m3u
    ##         return m3u 
        

    ## def xspf(self, fpath):
    ##     node = model.make_node(fpath)
        
    ##     #if str(node.__class__) == "pose.model.osbrowser.Directory":
    ##     if node.find_type() == "Directory":
    ##         node.scan_filetypes()
    ##         playlist = MediaList(node.sounds, item_type="Sound")
            
    ##         return str(playlist.to_xspf())
    ##         #return str(node.path)
    ##     else:
    ##         playlist = MediaList(source=node.path)
    ##         playlist.from_file()
    ##         playlist.multi_filter()
    ##         #return str("NO MATCH")
    ##         return str(playlist.to_xspf())

